# BEGIN COPYRIGHT
# *************************************************************************
# Licensed Materials - Property of IBM 
# 5724-L01, 5655-N53, 5724-I82, 5655-R15
# (C) Copyright IBM Corporation 2007, 2008, 2009. All rights reserved. 
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
# *************************************************************************
# END COPYRIGHT

# import related classes
import sys;
from java.lang import *;
import java.io.File as File;
import re;
import shutil;
import os;
import os.path;

def isEmptyString(value):
    stringValue = String(value);                            
    if (stringValue.length() != 0):
        return "False";
    else:
        return "True";
#endDef

def getObjectArray(value):
    return getArray(value, lineSeparator);
#endDef


def getArray(value, separator):
    objectArray = [];
    
    # @todo: do we have a space problem here?
    stringValue = String(value);
    if (stringValue.length() != 0):
        objectArray = value.split(separator);
    
    return objectArray;
#endDef

def getNamedValue(nameValuePairs, name):
    value = "";
    
    nameValuePairsArray = nameValuePairs.split(":");
    
    index = 0;
    while index < len(nameValuePairsArray):
        if nameValuePairsArray[index] == name and ((index + 1) < len(nameValuePairsArray)):
            value = nameValuePairsArray[index + 1];
            break;
        else:
            index = index + 2; 
    
    return value;
#endDef

def getCellContext():
    print ""
    cellContext = ""
    
    try:
        cellContext =  "/Cell:" + System.getProperty("local.cell");        
    except:
        print "Cannot get the cell context. The exception type is: ", sys.exc_type, ". The exception description is: ", sys.exc_value      
    
    print "The value of cellContext is ", cellContext
    print ""
    return cellContext
#endDef

def getNodeContext(userContext):
    print ""
    nodeContext = ""
    
    if (userContext == "use-local-server"):
        nodeName = System.getProperty("local.node");
    else:
        nodeName = getNamedValue(userContext, "nodes");
    
    if (isEmptyString(nodeName) == "False"):
        nodeContext =  "/Node:" + nodeName;
    
    print "The value of nodeContext is ", nodeContext
    print ""
    return nodeContext
#endDef

def getServerContext(userContext):
    print ""
    serverContext = ""
    
    if (userContext == "use-local-server"):
        serverName = getLocalServerName();
    else:
        serverName = getNamedValue(userContext, "servers");
    
    if (isEmptyString(serverName) == "False"):
        serverContext =  "/Server:" + serverName;
    
    print "The value of serverContext is ", serverContext
    print ""
    return serverContext
#endDef

def getClusterContext(userContext):
    print ""
    clusterContext = ""
    
    clusterName = getNamedValue(userContext, "clusters");
     
    if (isEmptyString(clusterName) == "False"):
        clusterContext =  "/ServerCluster:" + clusterName;
    
    print "The value of clusterContext is ", clusterContext
    print ""
    return clusterContext
#endDef

def isCluster(clusterContext):
    print ""   
    isCluster = "False";
    
    objectID = AdminConfig.getid(clusterContext);
    objectType = AdminConfig.getObjectType(objectID);
    print "The value of objectType is ", objectType
    
    if (objectType == "ServerCluster"):
        isCluster = "True";
    
    
    print "The value of isCluster is ", isCluster 
    print ""
    return isCluster;
#endDef

def getDeploymentTargetContext(userContext):
    print ""
    deploymentTargetContext = "";
    
    deploymentTargetContext =  getCellContext();
    
    clusterContext = getClusterContext(userContext);        
    if (isEmptyString(clusterContext) == "True"):
        nodeContext = getNodeContext(userContext);
        serverContext = getServerContext(userContext);
        if (isEmptyString(nodeContext) == "False" and isEmptyString(serverContext) == "False" ):
             deploymentTargetContext = deploymentTargetContext + nodeContext + serverContext;
        else:
            print "Cannot get deployment target context for ", userContext;
    else:
        deploymentTargetContext = deploymentTargetContext + clusterContext;
    
    print "The value of deploymentTargetContext is ", deploymentTargetContext
    print ""
    return deploymentTargetContext;
#endDef

def getDeploymentTargetContextAt(nodeName, serverName, clusterName):
    print ""
    deploymentTargetContext = "";
    
    cellContext = getCellContext();    
    
    if (not (clusterName)) or (isEmptyString(clusterName) == "True"):
    	serverContext = "/Server:" + serverName;
    	nodeContext = "/Node:" + nodeName;
        deploymentTargetContext = cellContext + nodeContext + serverContext;    
    else:
        clusterContext = "/ServerCluster:" + clusterName;
        deploymentTargetContext = cellContext + clusterContext;
        
    print "The value of deploymentTargetContext is ", deploymentTargetContext
    print ""
    return deploymentTargetContext;
#endDef

def getLocalServerName():
    print "";
    nodeID = AdminConfig.getid(getCellContext() + getNodeContext("use-local-server"));
    servers = getObjectArray(AdminConfig.list("Server", nodeID));
    serverName = "";    
    
    print "The value of servers is ", servers
    
    # Look for the DEPLOYMENT_MANAGER first
    for server in servers:
        servertype = AdminConfig.showAttribute(server, "serverType");
        if (servertype == "DEPLOYMENT_MANAGER"):
            print "servertype is ", servertype 
            serverName = AdminConfig.showAttribute(server, "name");
            break;
    
    #If DEPLOYMENT_MANAGER not found, look for the first APPLICATION_SERVER
    #assumption: for local server there would be only one application server on a local node. this is used only in stand-alone cell
    if( serverName == ""):     
        for server in servers:
            serverType = AdminConfig.showAttribute(server, "serverType");
            if (serverType == "APPLICATION_SERVER"):
                print "servertype is ", servertype
                serverName = AdminConfig.showAttribute(server, "name");
                break;
            
    print "The value of serverName is ", serverName;
    print "";
    return serverName;       
#endif

def getCellType():
    print ""
    cellType = ""
    
    try:
        print "getting cellID"
        cellID = AdminConfig.getid(getCellContext());
        print "cellID : ", cellID 
        
        print "getting cell type"
        cellType = AdminConfig.showAttribute(cellID, 'cellType');
        
    except:
        print "Cannot get the cell type. The exception type is: ", sys.exc_type, ". The exception description is: ", sys.exc_value      
    
    print "The value of cellType is ", cellType
    print ""
    return cellType
#endDef

def isDMgrNode():
    print ""
    isDMgrNode = "False"
    
    try:
        nodeNameParam = '[-nodeName ' + System.getProperty("local.node") + ']'
        
        print "The value of nodeNameParam is ", nodeNameParam
        servers = AdminTask.listServers(nodeNameParam).split(System.getProperty("line.separator"));
        print "The value of servers is ", servers
        
        for server in servers:
            servertype = AdminConfig.showAttribute(server, "serverType");
            print "The value of servertype is ", servertype 
            if (servertype == "DEPLOYMENT_MANAGER"):
                isDMgrNode = "True"
                break;
                    
    except: 
        print "Cannot find the node type. The exception type is: ", sys.exc_type, ". The exception description is: ", sys.exc_value
        
    print "The value of isDMgrNode is ", isDMgrNode 
    print ""
    return isDMgrNode               
#endDef

def getResourceEnvironmentProvider(deploymentTargetContext, providerName):
    print ""
    resourceEnvironmentProvider = "";
    
    dt = AdminConfig.getid(deploymentTargetContext);
    print "Getting resource provider named " ,providerName,  " at ",  dt;
    print ""
    
    resourceEnvironmentProviders = getObjectArray(AdminConfig.list("ResourceEnvironmentProvider", dt))
    
    print "Resource environment providers are ", resourceEnvironmentProviders
    print ""
    
    for currentResourceEnvironmentProvider in resourceEnvironmentProviders:
        name = AdminConfig.showAttribute(currentResourceEnvironmentProvider, "name");
        
        print "The resource environment provider name is ", name        
        if (name == providerName):
            resourceEnvironmentProvider = currentResourceEnvironmentProvider;
            break;
            
    print "The resource environment provider is ", resourceEnvironmentProvider;
    print ""   
    return resourceEnvironmentProvider;
                     
#endDef

# supports getting only one object of a given type and creates one if needed
def getConfigObject(deploymentTargetContext, clusterConfigRootObjectType, createClusterRootObject, serverConfigRootObjectType, createServerRootObject, configObjectType, create, defaultAttributes):
    print ""
    configObject = "";
    
    dt = AdminConfig.getid(deploymentTargetContext);
    print "Getting configuration object of type ", configObjectType, " from ", clusterConfigRootObjectType, " or " , serverConfigRootObjectType, " at ",  deploymentTargetContext;
    print ""
    
    configRootObjects = [];
    if (isCluster(deploymentTargetContext) == "True"): 
        configRootObjects = getObjectArray(AdminConfig.list(clusterConfigRootObjectType, dt))
        
        if (len(configRootObjects) == 0) and (createClusterRootObject == "True"):
            print "Creating configuration root object"
            AdminConfig.create(clusterConfigRootObjectType, dt, '');
            configRootObjects = getObjectArray(AdminConfig.list(clusterConfigRootObjectType, dt))
    else:
        configRootObjects = getObjectArray(AdminConfig.list(serverConfigRootObjectType, dt))
        
        if (len(configRootObjects) == 0) and (createRootObject == "True"):
            print "Creating configuration root object"
            AdminConfig.create(serverConfigRootObjectType, dt, '');
            configRootObjects = getObjectArray(AdminConfig.list(serverConfigRootObjectType, dt))

    print "The value of configRootObjects is ",  configRootObjects;

    if (len(configRootObjects) > 0): 
    
        configObjects = getObjectArray(AdminConfig.list(configObjectType, configRootObjects[0]))
        
        print "The value of configObjects is ", configObjects
        print ""
        
        if (len(configObjects) > 0):
            configObject = configObjects[0];
        elif (create == "True"):
            print "Creating Service Monitor Server configuration"
            configObject = AdminConfig.create(configObjectType, configRootObjects[0], defaultAttributes);
            print "Created Service Monitor Server configuration"
            
    print "The value of configObject is", configObject;
    print ""   
    return configObject;
                         
#endDef

# supports getting only one object of a given type and creates one if needed
def getCoreServerConfigObject(deploymentTargetContext, configTypeName, create, defaultAttributes):
    print ""
    print "Getting core server configuration object of type ", configTypeName, " at ",  deploymentTargetContext;
    print ""
    
    serverConfigObject = getConfigObject(deploymentTargetContext, "CoreServerClusterExtension", "True", "CoreServerExtension", "False", configTypeName, create, defaultAttributes);
            
    print "The core server configuration object is ", serverConfigObject;
    print ""   
    return serverConfigObject;                         
#endDef

def isApplicationInstalled(applicationName):
    print ""
    isInstalled = "False"
    try:    
        print "is Application installed'" + applicationName + "'"; 
        applications  = getObjectArray(AdminApp.list());
        print "The value of applications is ", applications;
        for application in applications:
            if (application == applicationName):
                isInstalled = "True";
                break;                
    except:
        print "Cannot check the application '" + applicationName + "' The exception type is: ", sys.exc_type, ". The exception description is: ", sys.exc_value;        
        print 
        
    print ""
    print "The value of isInstalled is ", isInstalled
    return isInstalled;
#endDef

###################################################
# isApplicationInstalledAt
#			Checks if the given application is deployed on the given deploymentTargetContext
#			return: "True" or "False"
###################################################
def isApplicationInstalledAt(applicationName, deploymentTargetContext):
    print ""
    isInstalled = "False"
    try:    
        print "is Application installed'" + applicationName + "'"; 
        applications  = getObjectArray(AdminApp.list());
        print "The value of applications is ", applications;
        
        for application in applications:
            
            #Check if the app is in the app list
            if (application == applicationName):
                
                #Get the ApplicationDeployment for the application and match for the deploymentTargetContext
                print ""
                appDeploymentsList = getObjectArray(AdminConfig.list("ApplicationDeployment"));
                    
                print "appDeploymentsList : ", appDeploymentsList
                
                for appDeployment in appDeploymentsList:
                    print ""
                    print "The value of appDeployment is ", appDeployment
                    
                    stringValueAppDeployment = String(appDeployment);
                    
                    # appDeployement is a string with path info of the deployed application
                    if (stringValueAppDeployment.indexOf(String(application)) >=0 ):
                        print "ApplicationDeployment ", stringValueAppDeployment, " contains ", application;
                        
                        deploymentTargetMappings = AdminConfig.list("DeploymentTargetMapping", appDeployment);
                        deploymentTargetMappingsList = [];
                        
                        stringValue = String(deploymentTargetMappings);
                        if (stringValue.length() != 0):
                            deploymentTargetMappingsList = getObjectArray(deploymentTargetMappings);
                        
                        for deploymentTargetMapping in deploymentTargetMappingsList:
                            print "deploymentTargetMapping = ",deploymentTargetMapping;
                            
                            target = AdminConfig.showAttribute(deploymentTargetMapping, "target");
                            targetType = AdminConfig.getObjectType(target);
                            
                            print "target = " + target ;
                            print "targetType = " + targetType;
                            
                            serverName = None;
                            nodeName = None;
                            clusterName = None;
                            
                            if (targetType == "ServerTarget"):
                                serverName = AdminConfig.showAttribute(target, "name");
                                nodeName = AdminConfig.showAttribute(target, "nodeName");                                
                            elif (targetType == "ClusteredTarget"):
                                clusterName = AdminConfig.showAttribute(target, "name");
                                
                            thisDeploymentTargetContext = getDeploymentTargetContextAt(nodeName, serverName, clusterName);
                            
                            print "thisDeployementTargetContext = ", thisDeploymentTargetContext
                            
                            if(thisDeploymentTargetContext == deploymentTargetContext):
                                isInstalled = "True";
                                break;
                        #endFor
                    #endIf
                    
                    if (isInstalled == "True"):
                        break;
                #endFor
            #endIf
            
            if(isInstalled == "True"):
                break;
        #endFor
         
    except:
        print "Cannot check the application '", applicationName, " at deploymentTargetContext ", deploymentTargetContext, "' The exception type is: ", sys.exc_type, ". The exception description is: ", sys.exc_value;        
        print 
        
    print ""
    print "The value of isInstalled is ", isInstalled
    return isInstalled;
#endDef

def uninstallApplication(applicationName):
    print ""
    try:    
        print "Trying to uninstall the application '" + applicationName + "'"; 
        AdminApp.uninstall(applicationName);
        print "Completed application uninstall for" + applicationName + "'";
    except:
        print "Cannot uninstall application '" + applicationName + "' The exception type is: ", sys.exc_type, ". The exception description is: ", sys.exc_value;        
        print 
    print ""
#endDef

def updateApplication(applicationName, contentType, options):
    print ""
    try:    
        print "Trying to update the application '" + applicationName + "' with content type " + contentType + " and options " + options; 
        AdminApp.update(applicationName, contentType, options) 
        print "Completed the application update for " + applicationName + "'";
    except:
        print "Cannot update application '" + applicationName + "' The exception type is: ", sys.exc_type, ". The exception description is: ", sys.exc_value;        
        print 
    print ""
#endDef

